/* twpsk  - PSK31 for Linux with a Lesstif interface
 * Copyright (C) 1999 Ted Williams WA0EIR (ted@bluestone.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Version: 1.1 - Aug 2000
 *
 * Portions of the file derived from:
 * Soundcard-based implementation of the PSK31 HF keyboard-to-keyboard mode
 * Copyright (C) 1998  Hansi Reiser, DL9RDZ
 */

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <linux/soundcard.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <termios.h>
#include <math.h>

#include "GUI.h"
#include "../server/server.h"
#include "twpskWids.h"
#include "twpskScope.h"
#include "twpskDisp.h"

int OPTnew = 0;   // TODO: rausdamit (-> filter and sync alg selection for RX)


#if 0
#define DMA_BUF_BITS 6
// Audio parameters
int full_duplex=0;
static int audio_fd;
#endif

static char *sound_device="/dev/audio";

/* Globals */
int transmit=0;
AppRes appRes;
Wids pskwids;
Scope scope;
Disp disp;
XtAppContext ac;

/* @@@ DL9RDZ */
#include "catWids.h"
#include "decoderWids.h"

Cat catwids;

DecoderWid decoderWids[MAX_DECODERS];

   
/*
 * main - main function for twpsk
 */
int main(int argc, char *argv[])
{

   Widget shell, temp;
   Widget  noTwpskMessBox;
   int val; 
   int height = 0, width = 0;
   XmString messStr;
   char *str = "";

   String fallback_resources[] =
     {
     "twpsk.noTwpskMessBox.forground:     black",
     "twpsk.noTwpskMessBox.background:    tan",
     "twpsk.noTwpskMessBox.messageString: Can't find the resource file, Twpsk.",
      NULL
      };

   XtResource appRes_desc[] =
   {
      {
      XmNtwpskDir,                       /* Resource Name */
      XmCTwpskDir,                       /* Resource Class */
      XmRString,                         /* Resource Data Type */
      sizeof (char),                     /* Size of Resource */
      XtOffsetOf (AppRes, twpskDir),     /* Offset into Struct */
      XtRString,                         /* Default Data Type */
      (XtPointer) "Bogus Name"           /* Default Value */
      },

      {
      XmNbuttonNames,                    /* Same as above */
      XmCButtonNames,
      XmRString,
      sizeof (char),
      XtOffsetOf (AppRes, buttonNames),
      XtRString,
      (XtPointer) "Bogus Name"
      },

      {
      XmNrxFreq,                         /* Same as above */
      XmCRxFreq,
      XmRFloat,
      sizeof (float),
      XtOffsetOf (AppRes, rxFreq),
      XtRFloat,
      (XtPointer) 0
      },

      {
      XmNtxFreq,                         /* Same as above */
      XmCTxFreq,
      XmRFloat,
      sizeof (float),
      XtOffsetOf (AppRes, txFreq),
      XtRFloat,
      (XtPointer) 0
      },

      {
      XmNafc,                            /* Same as above */
      XmCAfc,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, afc),
      XtRInt,
      (XtPointer) 0
      },

      {
      XmNnet,                            /* Same as above */
      XmCNet,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, net),
      XtRInt,
      (XtPointer) 0
      },

      {
      XmNnoMixer,                        /* Same as above */
      XmCNoMixer,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, noMixer),
      XtRInt,
      (XtPointer) 1
      },

      {
      XmNmainVol,                        /* Same as above */
      XmCMainVol,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, mainVol),
      XtRFloat,
      (XtPointer) 0
      },

      {
      XmNoutVol,                         /* Same as above */
      XmCOutVol,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, outVol),
      XtRFloat,
      (XtPointer) 0
      },

      {
      XmNinVol,                          /* Same as above */
      XmCInVol,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, inVol),
      XtRImmediate,
      (XtPointer) 0
      },

      {
      XmNcall,                           /* Same as above */
      XmCCall,
      XmRString,
      sizeof (char),
      XtOffsetOf (AppRes, call),
      XtRImmediate,
      (XtPointer) " ",
      },

      {
      XmNzero,                           /* Same as above */
      XmCZero,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, zero),
      XtRImmediate,
      (XtPointer) 0
      },

      {
      XmNptt,                            /* Same as above */
      XmCPtt,
      XmRInt,
      sizeof (int),
      XtOffsetOf (AppRes, ptt),
      XtRImmediate,
      (XtPointer) 0
      },

      {
      XmNserDev,                         /* Same as above */
      XmCSerDev,
      XmRString,
      sizeof (char),
      XtOffsetOf (AppRes, serDev),
      XtRImmediate,
      (XtPointer) "/dev/ttyS1",
      },
   };

   /*
    * Create the shell and read the resource file
    */
   shell = XtVaAppInitialize (&ac, "Twpsk", NULL, 0,
      &argc, argv,
      fallback_resources,
      XmNiconName, "TWPSK",
      NULL);
   /*
    * Create an error message box just in case.
    * If the label != "OK", then we're running on
    * the fallback resources.  Popup the dialog
    * and tell them to fix Twpsk
    */
   noTwpskMessBox = XtVaCreateManagedWidget("noTwpskMessBox",
      xmMessageBoxWidgetClass, shell, 
      NULL);

   XtVaGetValues (noTwpskMessBox,
      XmNmessageString, &messStr,
      NULL);

   XmStringGetLtoR (messStr, XmFONTLIST_DEFAULT_TAG, &str);

   if (strcmp(str,"OK") != 0)  /* didn't find Twpsk */
   {
      /* Get rid of the Cancel and Help buttons */
      temp = XmMessageBoxGetChild (noTwpskMessBox, XmDIALOG_CANCEL_BUTTON);
      XtUnmanageChild(temp);
      temp = XmMessageBoxGetChild (noTwpskMessBox, XmDIALOG_HELP_BUTTON);
      XtUnmanageChild(temp);

      /* Have the OK button call the quitCB */
      temp = XmMessageBoxGetChild (noTwpskMessBox, XmDIALOG_OK_BUTTON);
      XtAddCallback (temp, XmNactivateCallback, quitCB, (XtPointer)0);
      /* bail out to the event loop with just the message box */
      XtRealizeWidget(shell);
      XtAppMainLoop(ac);
   }

   /* Got Twpsk, so don't need the message box */
   XtDestroyWidget (noTwpskMessBox);
   XtGetApplicationResources (shell, &appRes, appRes_desc,
      XtNumber (appRes_desc), NULL, 0);

   /*
    * initialize server
    */
   if( server_main(sound_device, appRes.ptt?appRes.serDev:NULL, appRes.twpskDir) )
   {
	   fprintf(stderr,"failed to initialize psk31 server\n");
	   exit(1);
   }

   /*
    * Create widgets
    */
   pskwids.buildWidgets (shell, &appRes);  

   /* @@@DL9RDZ
    * test 2nd window
    */
   //catwids.buildWidgets(shell, &appRes);

   DecoderWid::shell = shell;
   for(int i=0; i<MAX_DECODERS; i++) {
       decoderWids[i].visible = -1;
   }

   /*
    * setup and initialize the scope, and the display
    */
   scope.setup (shell, pskwids.getScope());
   disp.setup(pskwids.getWF());
   
   /*
    * Configure mixer if desired
    */
   if (appRes.noMixer == 0)
   {
      int mixer_fd;
      mixer_fd = open("/dev/mixer",O_RDWR);
      if (mixer_fd < 0)
      {
         perror ("twcw - Can't open mixer");
         exit (1);
      }
      val=SOUND_MASK_LINE|SOUND_MASK_MIC;
      ioctl(mixer_fd,SOUND_MIXER_WRITE_RECSRC,&val);  
      close(mixer_fd);
   }
   else
      fprintf (stderr, "Mixer disabled\n");

   /*
    * Set min size
    */
   XtVaGetValues (shell,
      XmNwidth, &width,
      XmNheight, &height,
      NULL);

   XtVaSetValues (shell,
      XmNminWidth, width,
      XmNminHeight, height,
      NULL);
   
//    printf ("width = %d height = %d\n", width, height);    

   /*
    * Start the application work proc
    * and enter the main loop
    */
   XtAppAddWorkProc (ac, workProc, (XtPointer) NULL);
   XtAppMainLoop (ac); 
}


#ifndef USE_PTHREAD
void master_handler(void);
#endif

/*
 * workProcFunction
 * Work Proceedure for receive and transmit
 */
Boolean workProc(XtPointer cdata)
{
   PSK31info rxinfo, txinfo;
   char str[8];
   int dcd;
   static float rxfreqOld;
   int phdelta;  
   int l;   
   int rxecho;
   
#ifndef USE_PTHREAD
   // 4 is an empirical value.
   // If choosen too low, sound card buffer will run empty, and thus sound
   // will have interruptions (fatal!). A value too high isn't that critical...
   for(int i=0; i<4; i++) 
   {
      master_handler();
#endif
      // handle FFT!
      int N = disp.getsamplecnt();
      float fftval[N];
      l = commGetData(COMM_FFTCH, (char *)fftval, sizeof(fftval));
      if (l > 0)
      {
         if (l != N)
         {
            fprintf(stderr,"FFT len mismatch: l=%d N=%d\n",l,N);
         }
         disp.getFFT (fftval, N);
      }
#ifndef USE_PTHREAD
   }
#endif
   char buf[256];
   
   /*
    * handle transmit echo!  we could use a different color?
    */
   rxecho = commGetData(COMM_ECHOCH, buf, sizeof buf);
   if(rxecho > 0)
   {
      for(int i=0; i<rxecho; i++)
         appendRXtext(pskwids.getRxText(), buf[i], appRes.zero);
   }
   commGetInfo(COMM_TXCH, &txinfo, sizeof(txinfo));
   //(TODO) well I probably dont really need this info... 
   //txfreq = 0.01 * txinfo.freq;

   /*
    * handle receive!
    */
   l = commGetData(COMM_RXCH, buf, sizeof buf);
   if(l > 0)
   {
      for(int i=0; i<l; i++)
         appendRXtext(pskwids.getRxText(), buf[i], appRes.zero);
   }

   commGetInfo(COMM_RXCH, &rxinfo, sizeof(rxinfo));
   appRes.rxFreq = 0.01 * rxinfo.freq;
   phdelta = rxinfo.phdelta;
   
   /*update DCD toggle button*/
   dcd = rxinfo.dcd;
   XtVaSetValues(pskwids.getDcdTB(), XmNset, dcd, NULL);

   if(rxinfo.ptt == 0)
   {
      // receiving
      /* update scope */
      scope.drawline(rxinfo.phdelta, rxinfo.strength, rxinfo.dcd);

      /* update rx freq */
      /* but only if focusFlag == 0 */
      if (pskwids.getRxFreqFocus() == 0 )
      {
         if (fabs(appRes.rxFreq - rxfreqOld) > .05)
         {
            sprintf (str, "%4.1f", appRes.rxFreq);
            XmTextFieldSetString (pskwids.getRxFreqTF(), str);
            /* tell the fft about it */
            disp.offset(appRes.rxFreq);
            rxfreqOld = appRes.rxFreq;
         }
      }
   }
   else
   {
      /* scope display for each tx character put in the echo (rx) window*/
      if(rxecho>0)
         phdelta = 0;
      else
         phdelta = 128;
      scope.drawline(phdelta, 35, GREEN);
   }

   /* handle receive for additional receivers! */
   for(int ch=0; ch<MAX_DECODERS; ch++)
   {
      if( decoderWids[ch].visible!=1) continue;
         l = commGetData(ch+3, buf, sizeof buf);
      if(l>0)
      {
         for(int i=0; i<l; i++)
            appendRXtext(decoderWids[ch].getTextWid(), buf[i], appRes.zero);
      }
      l = commGetInfo(decoderWids[ch].commChannel, &rxinfo, sizeof(rxinfo));
      if(l==0)
      {
         decoderWids[ch].updateDisplay(0.01*rxinfo.freq, rxinfo.dcd, 0);
         decoderWids[ch].getScope().drawline(rxinfo.phdelta, 
            rxinfo.strength, rxinfo.dcd);
      }
   }

#ifdef USE_PTHREAD
   usleep(10000);
#else
   usleep(5000);
#endif

   return (False);
}
