/* twpsk  - A gui application for PSK
 * Copyright (C) 1999 Ted Williams WA0EIR (ted@bluestone.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Version: 1.1 - Aug 2000
 *
 * Disp Class - methods to draw the waterfall and spectrum analyser
 */

#include <stdio.h>
#include <math.h>
#include "twpskDisp.h"
#include "twpskWids.h"
#include "wfCursorShape.xbm"
#include "wfCursorMask.xbm"
#include "../server/server.h"

extern Wids pskwids;
extern AppRes appRes;

void Disp::find_visual_info(int *truecolor, XVisualInfo *vinfo)
{
   int scr = DefaultScreen(display);
   if( XMatchVisualInfo(display, scr, 8, PseudoColor, vinfo) )
   {
      fprintf(stderr,"Using 8 bpp Pseudo Color\n");
      *truecolor = 0; return;
   }
   if( XMatchVisualInfo(display, scr, 15, TrueColor, vinfo) )
   {
      fprintf(stderr,"Using 15 bpp True Color\n");
      *truecolor = 1; return;
   }
   if( XMatchVisualInfo(display, scr, 16, TrueColor, vinfo) )
   {
      fprintf(stderr,"Using 16 bpp True Color\n");
      *truecolor = 1; return;
   }
   if( XMatchVisualInfo(display, scr, 24, TrueColor, vinfo) )
   {
      fprintf(stderr,"Using 24 bpp True Color\n");
      *truecolor = 1; return;
   }
   if( XMatchVisualInfo(display, scr, 32, TrueColor, vinfo) )
   {
      fprintf(stderr,"Using 32 bpp True Color\n");
      *truecolor = 1; return;
   }
}


// color: 16 bit values! 0x0000..0xFFFF
unsigned long Disp::get_mask_color(int color, unsigned long mask)
{
   int shift = 0;
   int notused = 16;
   
   while (!(mask&0x01))
   {
      shift++;
      mask >>=1;
   }
   
   while(mask&0x01)
   {
      notused--;
      mask >>=1;
   }

  unsigned long c = (color>>notused) << shift;
  return c;
}


/*
 * setup for wf/sa drawing area
 */
void Disp::setup (Widget wfDA)
{
   /* constants */
   int i;
   int bad_grays = 0;
   int rtn = 0;

   /* window values */
   display = XtDisplay (wfDA);
   window  = XtWindow (wfDA);
   screen  = DefaultScreen (display);
   gc = XDefaultGC (display, screen);
   cmap = DefaultColormap (display, screen);
   int wfCursorHotX = 7;
   int wfCursorHotY = 15;
   XColor cursorShapePix, cursorMaskPix;
   Pixmap shape, mask;

   XSetLineAttributes (display, gc, LINE_WIDTH, LineSolid, CapRound, JoinRound);

   /* waterfall cursor */
   shape = XCreatePixmapFromBitmapData (display, window,
      wfCursorShape_bits, wfCursorShape_width, wfCursorShape_height, 1, 0, 1);

   mask = XCreatePixmapFromBitmapData (display, window,
      wfCursorMask_bits, wfCursorMask_width, wfCursorMask_height, 1, 0, 1);

   XParseColor (display, cmap, "red", &cursorShapePix);
   XParseColor (display, cmap, "black", &cursorMaskPix); /*not the best color*/
   
   wfCursor = XCreatePixmapCursor (display, shape, mask, 
         &cursorShapePix, &cursorMaskPix, wfCursorHotX, wfCursorHotY);

   XDefineCursor (display, XtWindow (wfDA), wfCursor);

   cnt = 0;
   row = 0;

   /*
    * create 100 Gray pixels - but make
    * the first one the black pixel so
    * no signal won't create a white out
    */

/*   dispPix[0].pixel = BlackPixel(display, screen); */    /* black pixel */


   XVisualInfo vinfo;
   int truecolor;
   find_visual_info(&truecolor, &vinfo);

   if(!truecolor)
   {
      /* 8bit pseudo color (with color map) */
      for (i=0; i<MAX_GRAYS; i++)                       /* and 99 grays */
      {
         if (XAllocColorCells (display, cmap, False, NULL, 0,
                               &dispPix[i].pixel, 1) == 0) 
         {
            bad_grays++;                                /* count failures */
            continue;
         }
         dispPix[i].flags = DoRed | DoGreen | DoBlue;
         dispPix[i].red = (i + START) * 256;            /* scale to 65K */
         dispPix[i].green = (i + START) * 256;
         dispPix[i].blue = (i + START) * 256;
	 
         XStoreColor (display, cmap, &dispPix[i]);
      }
      if (bad_grays != 0)                               /* tell me about it */
         printf ("Could not allocate %d %s\n",
                  bad_grays, bad_grays == 1 ? "color" : "colors");
       
   }
   else  /* 15, 16, 24 or 32 bpp true color */
   {
      int r,g,b;
      for(i=0; i<MAX_GRAYS; i++)
      {
         // r=g=b= (i+START)*256;
         // The previous line is equivalent to your pseudo color code,
         // but I personally prefer a bigger range of color intensity,
         // thus using 255,253,251,249,.. instead of 255,254,253,252,...
         r=g=b=(i*2+55)*256;
         dispPix[i].pixel = (get_mask_color(r, vinfo.red_mask) |
                             get_mask_color(g, vinfo.green_mask) |
                             get_mask_color(b, vinfo.blue_mask) );
      }
   }

   /* get pixel for red line */
   rtn = XAllocNamedColor (display, cmap, "red", &redPix, &defPix);

   /* Setup fft values */
   fft_setup (512 * pskwids.getFFTsample() , pskwids.getFFTspeed());    
}


/*
 * fft_setup method - set fft values for sample sizes
 * speed: 0..100 --> overlap=2048..256
 * 0 -> 2^11   100-> 2^8
 */
void Disp::fft_setup(int samps, int speed)
{
   static int fftspeed=0;
   if(samps > 0)
   {
      samples = samps;               /* Size of the sample - 512, 1K, 2K */
      // fprintf(stderr,"sample size: %d\n",samps);
   }
   if (speed >= 0)
   {
      if(speed > 100)
      {
         fprintf(stderr,"error: speed=%d\n",speed);
      }
      else
      {
         float expo = 8+(100-speed)/99.0*3;  // 8..11
         fftspeed = (int)pow(2.0, expo);
         // fprintf(stderr,"setting fft speed to %d\n",fftspeed);
      }
   }

   ffts = samples/2;                  /* Number of filters */
   deltaf = (float)8000 / samples;    /* Hz per ffts sample */

// printf ("freq C  %f\n", appRes.rxFreq);
   delta = (int)(fc / deltaf - DISPLAY_WIDTH/2);   /* Offset in ffts */
   cnt = 0;


// TJW  for IMD Calculations

   upperSig = DISPLAY_WIDTH/2 + (samples / 512);
   lowerSig = DISPLAY_WIDTH/2 - (samples / 512);

   upperIMD = DISPLAY_WIDTH/2 + (samples / 512) * 3;
   lowerIMD = DISPLAY_WIDTH/2 - (samples / 512) * 3;

   commControl(COMM_FFTCH, COMM_FFTN, samples);
   commControl(COMM_FFTCH, COMM_FFTOVERLAP, fftspeed);
}


// server handles fft data...
// when buffer full, this handler funciton in CB will scale and display...

void Disp::getFFT(float *val, int len)
{
   if(len != samples)
   {
      fprintf(stderr,"FFT length mismatch\n");
      // TODO: remove this error message. it may occure
      // when changing FFT parameters (really?)
   }
   IMDcalc(val);			/* calculate the IMD */
   mkPoints(val);			/* scale the fft data for display */
   drawDisplay();			/* draw the wf or sa */
}


/*
 * IMDcalc - calculates the IMD of the signal
 * If the fft values for both main sigs are greater than 5 (this blocks
 * the calc on no-signal and weak signal because the IMD is probably
 * not accurate), then calculate the IMD of each main sig, 
 * and add to IMDtotal.   After 20 pairs are colected, compute the average
 * and display.
 *
 * If there are 10 blocks in a row, put "---" in the widget, so we
 * don't display trash.
 */
void Disp::IMDcalc(float *val)
{
   static int cnt;
   static int block;
   static float IMDtotal = 0.0;
   static Widget imdTF = 0;
   char imdVal[8];
   float ratio;
  
   if (imdTF == 0)
      imdTF = pskwids.getImdTF();      /* get the IMD widget id */

   if (val[delta + lowerSig] > 5.0 && val[delta + upperSig] > 5.0)
   {
      cnt++;
      block = 0;

      ratio = val[delta + lowerIMD] / val[delta + lowerSig];
      IMDtotal = IMDtotal + 20.0 * log10 (ratio);
      
      ratio = val[delta + upperIMD] / val[delta + upperSig];
      IMDtotal = IMDtotal + 20.0 * log10 (ratio);

      if (cnt == 20)
      {
         sprintf (imdVal, "%2.0f\n", IMDtotal/40.0);
         XmTextFieldSetString (imdTF, imdVal);

         cnt = 0;
         IMDtotal = 0.0;
      }
   }
   else
   {
      block++;
      if (block == 10)
      {
         XmTextFieldSetString (imdTF, "---");
         block = 0;
      }
   }
}

/*
 * mkPoints method - gets DISPLAY_WIDTH floats from fft buffer,
 * scales with brightness, clip to max of 99, and put results in
 * values[x][y].
 */
void Disp::mkPoints(float *buffer)
{
   int x, y;
   float scale;

   /* fft data values seem to vary with sample size so,                 */
   /* samples/1024.0 -> 1,2,4 -  keeps the fft data values constant     */
   /* brightness / x -> try different values of x to find a good value  */

   scale = (brightness / 2.0) / (samples / 1024.0);   //TJW

   for (y=row, x=0; x<DISPLAY_WIDTH; x++)
   {
      if (x+delta<0)
         values[x][y]=0;
      else
         values[x][y] = (short)(buffer[x+delta] * scale);

      if (values[x][y] > 99)
         values[x][y] = 99;
   }
}


/*
 * drawDisplay method - draws display as spectrum or waterfall
 */
void Disp::drawDisplay()
{
   int x, y;
   int pixIndex;
   int r = row + 1;      /* For WF - start draw from oldest info */

   if (r == DISPLAY_HEIGHT)
      r = 0;

   switch (displayFlag)
   {
      /*
       * draw waterfall
       */
      case WF:
         for (y=0; y < DISPLAY_HEIGHT; y++)
         {
            for (x=0; x < DISPLAY_WIDTH; x++)
            {
               /* don't draw over the red line */
               if ((x <= (DISPLAY_WIDTH - LINE_WIDTH) / 2)
                  || (x >= (DISPLAY_WIDTH + LINE_WIDTH) / 2))
               {
                  pixIndex = (int)(values[x][r]);
                  XSetForeground (display, gc, dispPix[pixIndex].pixel);
                  XDrawPoint (display, window, gc, x, y);
               }
            }
            r++;
            if (r == DISPLAY_HEIGHT)   /* recirculate? */
               r = 0;
         }
         break;

      /*
       * draw spectrum analyser
       */
      case SA:
         XClearArea (display, window, 0, 0, DISPLAY_WIDTH, DISPLAY_HEIGHT, 0);
         XSetLineAttributes (display, gc, 1, LineSolid, CapRound, JoinRound);

         for (x=0; x<DISPLAY_WIDTH; x++)
         {
            pixIndex = (int)(0.333 * values[x][row]);

            XDrawLine (display, window, gc,
                       x, DISPLAY_HEIGHT,
                       x, DISPLAY_HEIGHT - pixIndex);
         }

         XSetLineAttributes (display, gc, LINE_WIDTH,
                             LineSolid, CapRound, JoinRound);
         break;

      default:
         fprintf (stderr,"twpskDisplay.drawDisplay - Broken WF/SA value\n");
         break;
   }

   /* Increment row and make it circulate */
   row++;
   if (row == DISPLAY_HEIGHT)
      row = 0;

   /* Finally, draw red centerline and restore the fg color */
   XSetForeground (display, gc, redPix.pixel);

   XDrawLine (display, window, gc,
              DISPLAY_WIDTH/2, 0, DISPLAY_WIDTH/2, DISPLAY_HEIGHT);

   XSetForeground (display, gc, WhitePixel(display, screen));
}


/*
 * offset method - sets delta to starting index value of values array
 */
void Disp::offset(float freq)
{
   fc = freq;
   delta = (int)(fc / deltaf - DISPLAY_WIDTH/2);
}


/*
 * new_fc method - given the cursor's x position in the display,
 * returns the frequency 
 */
float Disp::new_fc (int x)
{
   float newfreq;

   x = find_center(x);             /* find the center */
   newfreq = fc + deltaf * (x-DISPLAY_WIDTH/2);
   if(newfreq < 0 ) return 0;
   else if (newfreq >= 8000/2) return 8000/2;
   else return newfreq;
}

/*
 * find_center method - trys to find the center of the signal that
 * was clicked.  Returns original value if I can't figure it out.
 */

//#define DETECT_LOW  1.75 * avg
//#define DETECT_HIGH 0.25 * avg

int Disp::find_center (int x)
{
   int i, j;
   int width = (int)(31.25 / deltaf);
   int len = (int)(2 * width);
   int start = x - len;
   int end = x + len;
   int confused = True;
   int max = 0;
   int total = 0, c = 1, avg;
   int low=0, high=0 ;
   int r;
 
   r = row - 1;
   if (r < 0)
      row = DISPLAY_HEIGHT - 1;

#if 0
printf ("x=%d width=%d len=%d start=%d end=%d\n", x, width, len, start, end);
for (i=start; i!=end; i++)
   printf ("%3d ", i);
printf("\n");
for (i=start; i!=end; i++)
   printf ("%3d ", values[i][r]);
printf ("\n\n");
#endif

   /* find maximums */
   for (i=start; i!=end; i++)
   {
      if (i<0) 
         continue;
      if (values[i][r] >= max)
         max = values[i][r];
   }

   /* find the maximums */
   for (i=start; i!=end; i++)
   {
      if (i<0)
         continue;

      if (values[i][r] == max )
      {
         low = i;

         /* find high end */
         for (j=end; j>low; j--)
         {
            if (values[j][r] == max && j - i <= width)
            {
               high = j;
               confused = False;
               break;
            }
         }
         if (!confused)
            break;
      }
      if (!confused)
         break;
   }

// should be after find max
   /* average non maximums */
   for (i=start; i!=end; i++)
   {
      if (i<0)
         continue;
      if (values[i][r] != max)
      {
         total = total + values[i][r];
         c++;
      }
   }
   avg = total / c;

#if 0
   printf ("Avg = %d\n", avg);
   printf ("low = %d high = %d\n", low, high);
   printf ("center = %d\n", (high+low)/2);
   printf ("ptr freq = %f\n", (delta + x) * deltaf);
   printf ("freq = %f\n", (delta + (high+low) / 2) * deltaf);
#endif

   if (avg * 4 < max && low != high && !confused)
   {
      //printf ("found wid = %d\n", high - low);
      return (high+low) / 2;
   }
   else
   {
      //printf ("using pointer\n");
      return x;
   }
}

