/* twpsk  - A gui application for PSK
 * Copyright (C) 1999 Ted Williams WA0EIR (ted@bluestone.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Version: 1.1 - Aug 2000
 */

#ifndef TWPSKWIDS_INCLUDED
#define TWPSKWIDS_INCLUDED
#define MAX_BUTTONS 9
#define SPACING 10
#define ARROW_WIDTH 35
#define NUM_XMSTRINGS 8

#include "GUI.h"
#include "../server/server.h"

/*
 * Wids class
 */

class Wids {

   private:
      int i;
      XmString xs[5];
      int qpsk, lsb;
      char str[8];
      Arg args[20];  
      Pixmap saPixmap, wfPixmap;
      Pixel fg, bg;
      int fftSpeed;
      int fftSample;
      int rxFreqFocus;
      int rxScrollFlag;
      int brightness;

      Widget mainform;
      Widget   label1, menuBar, fileMenu, fileCas;
      Widget   controlsMenu, controlsCas;
      Widget     rxBtn, txBtn, tuneBtn, sep;
      Widget     wideBtn, mediumBtn, narrowBtn; 
      Widget     saBtn, wfBtn;
      Widget   setupMenu, setupCas;
      Widget      audioBtn, displayBtn;
      Widget   helpMenu, helpCas, aboutBtn, helpBtn;
      
      Widget   rxTextSW;
      Widget     rxText;
      Widget     clrRxPopup;

      Widget   txTextSW;
      Widget     txText;
      Widget     clrTxPopup;
      
      Widget   displayForm; 
      Widget     scopeDA, wfDA, dispLabel;
      Widget     bwRB;
      Widget       wideTB, mediumTB, narrowTB;
      Widget     displayRB;
      Widget       saTB, wfTB;
      
      Widget   controlsForm;
      Widget     rxLabel;
      Widget     rxComboForm;
      Widget       rxFreqTF, rxUpArrow,rxDnArrow, afcTB;
      Widget     txLabel;
      Widget     txComboForm;
      Widget       txFreqTF, txUpArrow,txDnArrow, netTB;
      Widget     trLabel;
      Widget     controlsRC;
      Widget       trRB;
      Widget         rxTB, txTB, tuneTB;
      Widget     modesRC;
      Widget       modeLabel, dcdTB, qpskTB, lsbTB, cwidTB;
      Widget       imdFrame, imdForm, imdTF, imdLabel;
      
      Widget   audioDiag;
      Widget     mainLabel, mainScale, outLabel, pcmScale;
      Widget     inputLabel, inScale, audioCloseBtn;
      
      Widget   displayDiag;
      Widget     brightLabel,  brightScale, fftLabel, fftScale, displayCloseBtn;
      
      Widget rxScrollBar;
      Widget rxtTB[3];
      

      
      /*
       * createButtons - creates buttons for fileCas from
       * resource value buttonNames
       */
      void createButtons (Widget shell, char *buttonNames);


   public:
      Widget get_mainform() { return mainform; }
    int getLSB(void) {
	int value;
	XtVaGetValues(lsbTB, XmNset, &value, NULL);
	return value;
    }
    
    void setLSB(int value) {
	XtVaSetValues(lsbTB, XmNset, value, NULL);
    }
    int getQPSK() {
	int value;
	XtVaGetValues(qpskTB, XmNset, &value, NULL);
	return value;
    }
    void setQPSK(int value) {
	XtVaSetValues(qpskTB, XmNset, value, NULL);
    }
    int getAFC() {
	  int value;
	  XtVaGetValues(afcTB, XmNset, &value, NULL);
	  return value;
    }
    void setAFC(int value) {
	XtVaSetValues(afcTB, XmNset, value, NULL);
    }
    float getFreq() {
	String str;
	XtVaGetValues (rxFreqTF, XmNvalue, &str, NULL);  /* Get the freq */
	float f = atof(str);                        /* convert to a float */
	//fprintf(stderr,"tW: get: %f\n",f);
	return f;
    }
    void setFreq(float freq) {
	//fprintf(stderr,"tW: set: %f\n",freq);
	sprintf(str, "%6.1f", freq);
	XtVaSetValues(rxFreqTF, XmNvalue, str, NULL);
	commControl(COMM_RXCH, COMM_FREQ, (int)(freq*100));
    }

   /*
    * buildWidgets - builds the interface
    */
   void buildWidgets (Widget shell, AppRes *appRes);


   /*
    * getScope method - returns widget id of scopeDA
    */
   Widget getScope(void) { return (scopeDA); }


   /*
    * getWF method - returns widget id of wfDA
    */
   Widget getWF(void) { return (wfDA); }


   /*
    * getRxText method - returns widget id of rxText
    */
   Widget getRxText(void) { return (rxText); }


   /*
    * getTxText method - returns widget id of txText
    */
   Widget getTxText(void) { return (txText); }


   /*
    * getDcdTB method - returns widget id of dcdTB
    */
   Widget getDcdTB(void) { return (dcdTB); }


   /*
    * getRxFreqTF method - returns widget id of rxFreqTF
    */
   Widget getRxFreqTF(void) { return (rxFreqTF); }


   /*
    * getTxFreqTF method - returns widget id of txFreqTF
    */
   Widget getTxFreqTF(void) { return (txFreqTF); }


   /*
    * getCwidTB method - returns widget id of cwidTB
    */
   Widget getCwidTB(void) { return (cwidTB); }


   /*
    * getImdTF method - returns widget id of imdTF
    */
   Widget getImdTF(void) { return (imdTF); }


   /*
    * getRxFreqFocus method - returns true if rxFreqTF has focus
    */
   int getRxFreqFocus(void) { return (rxFreqFocus); }


   /*
    * setRxFreqFocus method - sets rxFreqFocus to f
    */
   void setRxFreqFocus(int f) { rxFreqFocus = f; }


   /*
    * setRxScrollFlag method - sets rxScrollFlag to x
    */
   void setRxScrollFlag(int x) { rxScrollFlag = x; }


   /*
    * getRxScrollFlag method - returns rxScrollFlag
    */
   int getRxScrollFlag(void) { return (rxScrollFlag); }


   /*
    * getFFTspeed method - returns fftSpeed
    */
   int getFFTspeed(void) { return (fftSpeed); }


   /*
    * getFFTsample method - returns fftSample
    */
   int getFFTsample(void) { return (fftSample); }
   
   
   /*
    * get rx, tx, and tune TB array
    */
    Widget *get_rxtTB (void) { return (rxtTB); }

};
#endif
