/* twpsk:  PSK31 for Linux with a Lesstif interface
 * Copyright (C) 1999 Ted Williams WA0EIR (ted@bluestone.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Version: 1.1 - Aug 2000
 */

#include "GUI.h"
#include "twpskWids.h"
#include "twpskDisp.h"
#include "twpskScope.h"
#include "../server/server.h"
#include "decoderWids.h"

#include <Xm/XmP.h>

#define SCROLLING 1

extern AppRes appRes;
extern Wids pskwids;
extern Disp disp;
extern Scope scope;
extern int netmode;

enum { MODE_RX, MODE_TX, MODE_TUNE };
int rxtxmode=MODE_RX;
Pixel darkBG, lightBG;

/*
 * appendRXtext - not really a callback, but fit well here
 * adds characters to the end of the RX text widget
 * and puts cursor to the last position.  Called by workProc()
 */
 void appendRXtext (Widget w, char ch, int zero)
{
   char str[2];
   int lastPos;

   str[0] = ch;                                   /* Build a string */
   str[1] = '\0';

   XmTextDisableRedisplay (w);
   lastPos = XmTextGetLastPosition(w);

   if (str[0] == '0' && zero == 1)                /* Print zero with a slash? */
   {
      str[0] = '\330';
   }

   if (str[0] == 0x7F || str[0] == '\b')          /* If a DEL or BS char */
   {
      XmTextReplace (w, lastPos-1, lastPos, "\0");
   }
   else
   {
      XmTextInsert (w, lastPos, str);
   }

   /* set the objects rxScrollFlag field */ 
   /* first, get the object */
   XtPointer *obj = NULL;
   XtVaGetValues (w,
      XmNuserData, &obj,
      NULL);

   if (obj == (XtPointer)&pskwids)                 /* for main window */
   {
      if (pskwids.getRxScrollFlag() != SCROLLING)  /* put cursor at end? */
      {
         XmTextShowPosition (w, lastPos);
      }
   }   
   else                                            /* for secondary windows */
   {
      DecoderWid *dec = (DecoderWid *) obj;

      if (dec->getRxScrollFlag() != SCROLLING)     /* put cursor at end? */
         XmTextShowPosition (w, lastPos);
   }
   XmTextEnableRedisplay (w);
}


/*
 * appendTXtext - not really a callback, but fit well here
 * adds characters to the end of the TX text widget
 * Called by fileCB()
 */
void appendTXtext (Widget w, char ch, int zero)
{
   char str[2];
   int lastPos;

   str[0] = ch;                                /* Build a string */
   str[1] = '\0';

   if (str[0] == '0' && zero == 1)             /* Print zero with a slash? */
   {
      str[0] = '\330';
   }

   lastPos = XmTextGetLastPosition(w);
   XmTextInsert (w, lastPos, str); 
   XmTextShowPosition (w, lastPos);
   XmTextSetInsertionPosition (w, lastPos);
}

/*
 * Callbacks
 */
 
/*
 * controlsBtnCB - callback for Recv, Xmit, Tune,
 * wide, medium, and narrow main menu buttons
 * Set the toggle button and let the valueChanged 
 * callback do the rest.
 */
void controlsBtnCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   Widget wid = (Widget)cdata;
   XmToggleButtonSetState (wid, True, True);   
}


/*
 * tuneCB - value changed callback for tune toggle button
 */
void tuneCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   if(rxtxmode == MODE_TUNE)
      /* Already receiving, i.e. nothing to do... */
      return;

   commControl(COMM_TXCH, COMM_MODE, MO_TUNE );
   
   if(rxtxmode == MODE_RX)
      commControl(COMM_TXCH, COMM_PTT, PTTON | PTTFORCE);

   rxtxmode = MODE_TUNE;
   changeBG ();
}


 /*
  * trCB - value changed callback for Recv/Xmit radio box togglebuttons
  * 'R' and 'T' are passed in the client data.  
  */
void rxCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   char cwStr[20];
   int phase;

   /* Going into receive mode */
   if( rxtxmode == MODE_RX )
   {
      /* Already receiving, i.e. nothing to do... */
      /* Maybe we should "force" rx if we are still transmitting? */
      return;
   }

   /* if CW ID needed? */ 
   if (rxtxmode!=MODE_TUNE && 
      (XmToggleButtonGetState(pskwids.getCwidTB()) == 1) )
      {
         strcpy (cwStr, " de ");                 /* Build cw id string */
         strncat (cwStr, appRes.call, 15);
         phase = 0;
         scope.drawline(phase, 35, GREEN);
         // switch to CW; PTT is already on
         commControl(COMM_TXCH, COMM_MODE, MO_CWSEND);
         commPutData(cwStr, 0);
      }
   // and send Postamble+schedule PTT off
   commControl(COMM_TXCH, COMM_PTT, PTTOFF);
   // turn off CW/Tune mode if necesary
   commControl(COMM_TXCH, COMM_MODE, MO_NORMAL);
   phase = 0;
   scope.drawline(phase, 35, GREEN);

   /* Now force focus to the xmit Text Widget */
   XmProcessTraversal (pskwids.getTxText(), XmTRAVERSE_CURRENT);
   rxtxmode = MODE_RX;
   changeBG ();
}


/*
 * txCB Callback
 */
void txCB (Widget w, XtPointer cdata, XtPointer cbs) 
{
   String str;
   int freq;
   int phase;

   /* Going into transmit mode */
   if(rxtxmode==MODE_TX)
   {
      /* Already transmitting, nothing to do */
      /* Maybe we should FORCE TX here (if we do a DCD schedule TX
       * otherwise) ?  Then we would have to take care about two
       * TX commands in the queue! (Maybe server/psk31tx does not yet
       * handle this correctly
       */
      return;
   }

   /* Going to tx from rx or tune mode */
   if(netmode)
   {
      XtVaGetValues (pskwids.getRxFreqTF(),      /* get the rx freq */
         XmNvalue, &str,
         NULL);

      XmTextFieldSetString (pskwids.getTxFreqTF(), str);   

      freq = atoi(str);
      commControl(COMM_TXCH, COMM_FREQ, (int)(100*freq));
   }

   phase = 128;
   scope.drawline(phase, 35, GREEN);

   if(rxtxmode==MODE_RX)
   {
      commControl(COMM_TXCH, COMM_MODE, MO_NORMAL);
      commControl(COMM_TXCH, COMM_PTT, PTTON);
   }
   else
   {
      // disable TUNE
      commControl(COMM_TXCH, COMM_MODE, MO_NORMAL);
   }

   /* Now force focus to the xmit Text Widget */ 
   XmProcessTraversal (pskwids.getTxText(), XmTRAVERSE_CURRENT);
   rxtxmode = MODE_TX;
   changeBG ();
}  


/*
 * changeBG - changes widget w's background to light or dark
 */
void changeBG (void)
{
   int i;
   Widget *w;
   
   w = pskwids.get_rxtTB();

   for (i=0; i<3; i++)
   {
      if (XmToggleButtonGetState(w[i]) == 0)
      {
         XtVaSetValues (w[i],
            XmNbackground, lightBG,
            NULL);
      }
      else
      {
         XtVaSetValues (w[i],
            XmNbackground, darkBG,
            NULL);
      }
   }
}   
   
/*
 * changeDisplayCB arm callback - changes display from spectrum to waterfall
 */
void changeDisplayCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   disp.changeDisplay ((int)cdata);
}


/*
 * txTextCB - modify/verify callback for transmit text widget
 */
void txTextCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XmTextVerifyCallbackStruct *pt = (XmTextVerifyCallbackStruct *) cbs;
   int i;
   // int phase;
   /* modify/verify Callback
    * handle DEL and Backspace chars. 
    */
   if (pt->reason == XmCR_MODIFYING_TEXT_VALUE)
   {   
      if (pt->text->length == 0)                /* Got a backspace */
      {  
         commPutData("\b", 1);
      }
      else
      {
         for (i = 0; i < pt->text->length; i++)
         {
            if (pt->text->ptr[i] == 0)
            {
               commPutData("\x7F", 1);
            }
            else
            {
               commPutData(pt->text->ptr+i, 1);
            }

            /* Print zero with a slash? */
            if (pt->text->ptr[i] == '0' && appRes.zero == 1 )
               pt->text->ptr[i] = '\330';
         }
      }
   }
}


/*
 * arrowCB - handles the Up/Down arrows for Rx and Tx freq changes.
 * The cdata is the associated textfield widget and userData indicates
 * 'R' or 'T' for recv or xmit freq.
 */
void arrowCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   float freq, delta;
   XtPointer userData;
   String str;
   unsigned char direction;
   Widget textF = (Widget)cdata;
   XmArrowButtonCallbackStruct *cbsPtr = (XmArrowButtonCallbackStruct *)cbs;
   XButtonPressedEvent *event = (XButtonPressedEvent *) cbsPtr->event;

   XtVaGetValues (w,
      XmNuserData, &userData,
      XmNarrowDirection, &direction,
      NULL);

   if (event->state & ShiftMask)            /* if shifted click */
      delta = 8.0;                          /* change by 8 */
   else 
      delta = 1.0;                          /* else change by 1 */

   if (cbsPtr->click_count == 2)            /* if double click, changes by 7  */
      delta = 7.0;                          /* cuz already been changed by 1 */

   XtVaGetValues (textF,                    /* Get the freq */
      XmNvalue, &str,
      NULL);
   freq = atof(str);                        /* convert to a float */

   if (direction == XmARROW_UP)
      freq = freq + delta;
   else
      freq = freq - delta;
      
   sprintf (str, "%4.1f", freq);            /* float to a char */
   
   XmTextFieldSetString (textF, str);

   if ((char)userData == 'R')      /* Set new freq in rx or tx class */
   {
           /* Set new rx freq or */
	   commControl(COMM_RXCH, COMM_FREQ, (int)(freq*100));
   }
   else
   {
           /* Set new tx freq */	   
	   commControl(COMM_TXCH, COMM_FREQ, (int)(freq*100));
   }
}


/*
 * freqTextCB - handles keyboard mod/ver and value changed callbacks
 * for rx and tx freq textfields.  userData is 'R' or 'T' for rx or tx
 * text widget or pointer to DecoderWid object for subwindows
 */
void freqTextCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   int i;
   float freq;
   String str;
   XtPointer userData;
   XmTextVerifyCallbackStruct *pt = (XmTextVerifyCallbackStruct *)cbs;

   /* Switch on the type of callback */   
   switch (pt->reason)
   {
      /* Activate Callback */
      /* if you type in a new freq */
      case XmCR_ACTIVATE:
         XtVaGetValues (w,
            XmNuserData, &userData,
            XmNvalue, &str,
            NULL);
            
         freq = atof(str);
          
         if ((char)userData == 'R')
         {
            /* Set new rx freq */
            commControl(COMM_RXCH, COMM_FREQ, (int)(freq*100));      
	    /* Now force focus to the xmit Text Widget */
	    XmProcessTraversal (pskwids.getTxText(), XmTRAVERSE_CURRENT);
         }
         else if ((char)userData == 'T')
         {
            /* Set new tx freq */
            commControl(COMM_TXCH, COMM_FREQ, (int)(freq*100));
	    /* Now force focus to the xmit Text Widget */
	    XmProcessTraversal (pskwids.getTxText(), XmTRAVERSE_CURRENT);
         }
	 else {
	    DecoderWid *dec = (DecoderWid *)cdata;
	    if(dec==NULL) {
		    fprintf(stderr,"callback with cdata==NULL and userData="
			    "%p (%c)\n", userData, (char)userData);
		    break;
	    }
	    commControl(dec->commChannel, COMM_FREQ, (int)(freq*100));
	    /* Now force focus to the Swap Button Widget */
	    XmProcessTraversal(dec->getSwapBtn(), XmTRAVERSE_CURRENT);
	 }

         break;

      /* Modify/Verify Callback */
      /* allow only digits or "." in freq text widgets */
      case XmCR_MODIFYING_TEXT_VALUE:
         for (i = 0; i < pt->text->length; i++)
         {   
            if (isdigit (pt->text->ptr[i]) || (pt->text->ptr[i] == '.'))   
               pt->doit = True;
            else
               pt->doit = False;
         }
         break;

      default:
         printf ("freqTextCB - unexpected callback type\n");
         break;      
   }
}


/*
 * afcCB - value changed callback - sets/clears the afc mode 
 */
void afcCB (Widget /*w*/, XtPointer /*cdata*/, XtPointer cbs)
{
   XmToggleButtonCallbackStruct *ptr = (XmToggleButtonCallbackStruct *) cbs;

   /* set AFC to toggle button state */
   commControl(COMM_RXCH, COMM_AFC, ptr->set);
}


/*
 * netCB - sets/clears the net mode 
 * should xfer rx freq to tx freq too???
 */
void netCB (Widget /*w*/, XtPointer /*cdata*/, XtPointer cbs)
{
   XmToggleButtonCallbackStruct *pt = (XmToggleButtonCallbackStruct *) cbs;

   netmode = pt->set;
}


/*
 * scopeCB - expose callback 
 */
void scopeCB (Widget /*w*/, XtPointer /*cdata*/, XtPointer /*cbs*/)
{
   scope.drawcirc();
}


/*
 * wfCB callback - click to tune
 * uses button release event and pointer x position to change rx freq
 */
void wfCB (Widget /*w*/, XtPointer /*cdata*/, XtPointer cbs)
{
   float  newfreq;
   char freq_str[8];
   int x;
   XmDrawingAreaCallbackStruct *ptr = (XmDrawingAreaCallbackStruct *) cbs;
   int button_number = ptr->event->xbutton.button;
   
#if 0
   if ( ptr->event->xbutton.button == 3)
   {
      printf ("%d\n\n", ptr->event->xbutton.x);
      return;
   }
#endif
   if (ptr->event->xany.type == ButtonPress)
   {
      x = ptr->event->xbutton.x;

      switch (button_number)
      {
         case Button1:
            newfreq = disp.new_fc(x);
	    sprintf(freq_str, "%4.1f", newfreq);
	    XmTextFieldSetString (pskwids.getRxFreqTF(), freq_str);
	    /* Set new rx freq */
            commControl(COMM_RXCH, COMM_FREQ, (int)(newfreq*100));
            break;

         case Button3:
	    // create new RX decoder!
	    float freq = disp.new_fc (x);
	    int ch;
	    for(ch=0; ch<MAX_DECODERS; ch++)
            {
               if (decoderWids[ch].visible == -1)
               {
                  decoderWids[ch].buildWidgets(DecoderWid::shell,&appRes);
                  break;
               } 
               else
                  if (decoderWids[ch].visible==0)
                     break;
            }

            if (ch >= MAX_DECODERS)
               fprintf(stderr, "error: no free decoder channel!\n");
            else
            {
               int channel = ch+3;
	       decoderWids[ch].commChannel = channel; // redundand
	       commControl(channel, COMM_MODE, MO_NORMAL); // init channel
	       commControl(channel, COMM_DCDLEVEL, 15);
	       /* start new sub windows with QPSK=off LSB=off and AFC=on */
	       commControl(channel, COMM_QPSK, 0);
	       commControl(channel, COMM_LSB, 0);
	       commControl(channel, COMM_AFC, 1);
	       decoderWids[ch].setControls(0, 0, 1);
	       /* set frequency... */
	       commControl(channel, COMM_FREQ, (int)(freq*100)); // set freq
	       decoderWids[ch].updateDisplay(freq,0,0);
	       /* clear text content! */
	       XmTextSetString(decoderWids[ch].getTextWid(), "");

	       decoderWids[ch].visible = 1;
	       int x,y,w;
	       XtVaGetValues(XtParent(pskwids.get_mainform()), 
			     XmNx, &x, XmNy, &y, XmNwidth, &w, NULL);
	       decoderWids[ch].move(x+w+4, y+ch*197);
	       XtPopup (decoderWids[ch].getMainWid(), XtGrabNone );
            }
      }
   }
}


/*
 * audioCB - sets audio level for main, line in, and line out on mixer
 * client data is 'm', 'i', and 'o' for main, line in, and line out
 */
void audioCB (Widget /*w*/, XtPointer cdata, XtPointer cbs)
{
   char which = (char) cdata;
   XmScaleCallbackStruct *ptr = (XmScaleCallbackStruct *)cbs;
   /* set stereo levels */
   int level = (ptr->value<<8) + ptr->value;

   switch (which)
   {
      case 'm':
         setLevel (SOUND_MIXER_VOLUME, level);
         break;

      case 'p':
         setLevel (SOUND_MIXER_PCM, level);
         break;
         
      case 'i':
         setLevel (SOUND_MIXER_LINE, level);
         setLevel (SOUND_MIXER_MIC, level);
         break;
   }
}   


/*
 * setLevel - sets level in mixer 
 * used by this callback and build_widgets() for initialization 
 */
void setLevel(int device, int level)
{
   int mixer_fd;

   /* Open the Mixer Device */
   if ((mixer_fd = open ("/dev/mixer", O_RDWR)) < 0)
   {
      fprintf (stderr, "Can't open mixer\n");
      perror ("Error: mixer open");
   }

   /* write new values to device */    
   if (ioctl (mixer_fd, MIXER_WRITE (device), &level) < 0)
   {
      perror ("Error: mixer write");
      close (mixer_fd);
   }
   close (mixer_fd);
}


/*
 * dcdCB - Squelch value changed callback 
 * sets dcd to opposite of button state(set)
 */
void dcdCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XmToggleButtonCallbackStruct *ptr = (XmToggleButtonCallbackStruct *) cbs;
   int set = ptr->set;

   commControl(COMM_RXCH, COMM_DCD, !set);
}


/*
 * fftScaleCB - handles FFT speed
 */
void fftScaleCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XmScaleCallbackStruct *ptr = (XmScaleCallbackStruct *)cbs;

   int val = ptr->value;
   disp.fft_setup(-1, val);
}


/*
 * qpskCB - enable/disable qpsk mode
 */
void qpskCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XmToggleButtonCallbackStruct *ptr = (XmToggleButtonCallbackStruct *) cbs;

   commControl(COMM_RXCH, COMM_QPSK, ptr->set);
   commControl(COMM_TXCH, COMM_QPSK, ptr->set);
}


/*
 * lsbCB - enable/disable lsb
 */
void lsbCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XmToggleButtonCallbackStruct *ptr = (XmToggleButtonCallbackStruct *) cbs;

   commControl(COMM_RXCH, COMM_LSB, ptr->set);
   commControl(COMM_TXCH, COMM_LSB, ptr->set);
}


/*
 * bwCB - changed bandwidth displayed
 */
void bwCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   int samp = (int) cdata;
   XmToggleButtonCallbackStruct *pt = (XmToggleButtonCallbackStruct *) cbs;

   if (pt->set == True)
   {
      disp.set_samples(samp);
   }
}


/*
 * fileCB - main menu file buttons
 */
void fileCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   char *fileName, *filePath;
   char *str, *s;
   Widget txTextWid;

   XtVaGetValues (w,
      XmNuserData, &fileName,
      NULL);

   filePath = (char *) XtMalloc (strlen(appRes.twpskDir) + strlen(fileName) + 2);
   strcpy (filePath, appRes.twpskDir);
   
   if (filePath[strlen (filePath) - 1] != '/')
      strcat(filePath, "/");

   strcat (filePath, fileName);

   /* read the file */
   str = getFile(filePath);

   txTextWid = pskwids.getTxText();                
   s = str;

   while (*s != '\0')
   {
      /* fix for sending double char - ver 88 (and earlier, I guess)    */
      /* had a broken XmTextInsert which didn't call the modify/verify  */
      /* callback.  If version <= 0.88, call commPutData here, because  */
      /* the modify/verify callback isn't called.  If ver > 0.88, don't */
      /* call commPutData here, because the callback will do it.  This  */
      /* Then, using the #define values from Xm.h, ... */

      if (LESSTIF_VERSION == 0 && LESSTIF_REVISION <= 88) 
         commPutData(s,1);
      appendTXtext (txTextWid, *s, appRes.zero);
      s++;
   } 
   XtFree (filePath);                /* Clean house */
   free (str);
}


/*
 * quitCB - QRT the program
 */
void quitCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   exit (0);
}


/*
 * rxFreqFocusCB - rxFreqTF focus callback
 * used by main window and subwindows!
 */
void rxFreqFocusCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   int focusFlag = 0;
   XmAnyCallbackStruct *ptr = (XmAnyCallbackStruct *) cbs;

   switch (ptr->reason)
   {
      case XmCR_FOCUS:
         XtVaSetValues (w,
            XmNforeground, WhitePixelOfScreen(XtScreen(w)),
            NULL);
         focusFlag = 1;
         break;

      case XmCR_LOSING_FOCUS:
         XtVaSetValues (w,
            XmNforeground, BlackPixelOfScreen(XtScreen(w)),
            NULL);
         focusFlag = 0;
         break;

      default:
         printf ("rxFreqFocusCB: gain/losing focus error\n");
         break;
   }

   /* set focusFlag in main or 2nd rx win object */
   // if(cdata==NULL || cdata==(XtPointer)&pskwids)
   if (cdata==(XtPointer)&pskwids)
      pskwids.setRxFreqFocus(focusFlag);
   else
   {
      DecoderWid *dec = (DecoderWid *)cdata;
      dec->setFreqFocus(focusFlag);
   }
}


/*
 * rxScrollBarCB - clears/sets rxScrollFlag to
 * enable/disable rxText scrolling
 */
void rxScrollBarCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   int max, position, slider;
   int scrollMode = 0;
   Widget rxText;
   DecoderWid *dec = (DecoderWid *)(cdata);

   /* Get the rxText widget of the Object that called us! */
   if (cdata == (XtPointer)&pskwids)  /* for the main window */
      rxText = pskwids.getRxText();      
   else                               /* for the secondary windows */
      rxText = dec->getRxText();

   /* get some values from the scrollbar */
   XtVaGetValues (w,
      XmNmaximum, &max,
      XmNvalue, &position,
      XmNsliderSize, &slider,
      NULL);

   /* Setup scroll mode off */
   if (max - position - slider == 0) 
   {  
      XtVaSetValues (rxText,   /* Go to normal mode if at the botton */
         XmNbackground, lightBG,
         NULL);

      /* These make the letters bg the same color as the widget's bg */
      XmTextSetHighlight (rxText, 0, 1, XmHIGHLIGHT_SELECTED);
      XmTextSetHighlight (rxText, 0, 1, XmHIGHLIGHT_NORMAL);
      
      scrollMode = 0;
   }

   /* Setup scroll mode on */
   if (max - position - slider != 0) // && scrollMode == SCROLLING)
   {
      XtVaSetValues (rxText,
         XmNbackground, darkBG,
         NULL);
      /* These make the letters bg the same color as the widget's bg */
      XmTextSetHighlight (rxText, 0, 1, XmHIGHLIGHT_SELECTED);
      XmTextDisableRedisplay (rxText);
      XmTextSetHighlight (rxText, 0, 1, XmHIGHLIGHT_NORMAL);    
      XmTextEnableRedisplay (rxText);

      scrollMode = 1;
   }
   /* Set scroll mode to scrollMode */
   if (cdata == (XtPointer)&pskwids)
   {
      pskwids.setRxScrollFlag (scrollMode);    /* for the main window */
   }
   else
   {
      dec->setRxScrollFlag (scrollMode);       /* for the secondary windows */
   }
}


/*
 * brightnessCB - sets disp.brightness equal to scale value
 */
void brightCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XmScaleCallbackStruct *ptr = (XmScaleCallbackStruct *)cbs;
   
   disp.brightness = ptr->value;
}


/*
 * popupDiagCB - pops up the setup Dialog
 */
void popupDiagCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   Widget form6 = (Widget) cdata;

   XtManageChild (form6);
}


/*
 * closeDiagCB - pops up the setup Dialog
 */
void closeDiagCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   Widget form6 = (Widget) cdata;

   XtUnmanageChild (form6);
}

/*
 * popupHandler - Event handler to popup clear text menu
 */

void popupHandler (Widget w, XtPointer cdata, XEvent *event,  Boolean *cont)
{
   Widget menuWid = (Widget)cdata;

   if (event->type == ButtonPress && event->xbutton.button == Button3)
   {
      XmMenuPosition (menuWid, &(event->xbutton));
      XtManageChild (menuWid);
   }
}


/*
 * clrTextCB - clears the text in the Rx and Tx window
 */

void clrTextCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   Widget textWid;

   XtVaGetValues (XtParent(w),       /* Get rowColumn (menu) wid */
      XmNuserData, &textWid,         /* userData is wid of text widget */
      NULL);

   XmTextSetString (textWid, "");
}


